// Copyright (C) 1999 Jean-Marc Valin & Dominic Letourneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/*
** Includes
*/
#include "Mux.h"

DECLARE_NODE(Mux)
/*Node

 * @name Mux
 * @category Logic
 * @description Multiplexer

 * @input_name SWITCH
 * @input_description The name of the input we want to pass through
 * @input_type string

 * @input_name DATA1
 * @input_description The First input (Object)

 * @input_name DATA2
 * @input_description The Second input (Object)

 * @output_name DATA
 * @output_description The input selected (Object)

END*/


Mux::Mux(string nodeName, ParameterSet params) 
   : Node(nodeName, params) 
{}


void Mux::specificInitialize()
{ 
   this->Node::specificInitialize();
}

int Mux::translateInput (string inputName)
{
  if(nameLookup.count(inputName) == 0)
    {
      nameLookup[inputName] = nameLookup.size();
    }

  return nameLookup[inputName];
}

int Mux::translateOutput (string outputName)
{
  if(outputName != "OUTPUT")
    {
      throw new NodeException(this, string("Unknown output name: ") + outputName, __FILE__, __LINE__);
    }

  return 0;
}

bool Mux::hasOutput(int output_id) const
{
  return(output_id == 0);
}

ObjectRef Mux::getOutput(int output_id, int count)
{
  if(output_id != 0)
    {
      throw new NodeException(this, string("Mux node does not have and output other than 0."), __FILE__, __LINE__);      
    }

  if(nameLookup.count("SWITCH") == 0)
    {
      throw new NodeException(this, string("Mux node needs a SWITCH input."), __FILE__, __LINE__);      
    }
  

  String switch_string;
  ObjectRef inputResults, switch_inputResults;
  NodeInput switch_input = inputs[nameLookup["SWITCH"]];

  // Start by getting the output of the switch input...
  if(nameLookup.count("SWITCH") == 0)
    {
      throw new NodeException(this, string("Mux node needs a SWITCH input."), __FILE__, __LINE__);      
    }
  switch_inputResults = switch_input.node->getOutput(switch_input.outputID,count);
  try
    {
      switch_string = object_cast<String>(switch_inputResults);
    }
  catch(GenericCastException& e)
    {
      e.print(cerr);
      throw new NodeException(this,string("Mux node SWITCH input must be a string."),__FILE__,__LINE__);
    }

  // Now lets get the output specified by the switch string
  if(nameLookup.count(switch_string) == 0)
    {
      throw new NodeException(this,string("Mux node does not have an input for switch ")+switch_string,__FILE__,__LINE__);
    }
  NodeInput input = inputs[nameLookup[switch_string]];
  inputResults = input.node->getOutput(input.outputID,count);


  return inputResults;
}
